unit Unit1;
interface
uses
  Windows, Messages, SysUtils, Classes, Graphics, Controls, Forms,
  Dialogs, StdCtrls, ExtCtrls, Math,
  Unit2, Spin;


const
LARGURA=15;
COMPRIMENTO=25;

type
TPanels = array of array of TPanel; //Tipo para mostrar o mapa do problema na tela

TAdversario = class(TComponent) //Classe que representa os adversrios
public
  X: integer;//Posio X do adversario  (coluna)
  Y: integer;//Posicao Y do adversario (linha)
  DX: integer;//Direao do movimento do adversario
  DY: integer;
  constructor Create(AOwner: TComponent); override;
  destructor Destroy; override;
end;

TEstado = class(TComponent) //Classe que armazena o estado do problema
public
  X: integer;//Posio X do Robo  (coluna)
  Y: integer;//Posicao Y do Robo (linha)
  constructor Create(AOwner: TComponent); override;
  destructor Destroy; override;
end;

TBola = class(TComponent) //Classe que armazena o estado do problema
public
  X: integer;//Posio X da Bola  (coluna)
  Y: integer;//Posicao Y da Bola (linha)
  DX: integer;//Direao do movimento da bola
  DY: integer;
  constructor Create(AOwner: TComponent); override;
  destructor Destroy; override;
end;

TGoleiro = class(TComponent) //Classe que armazena as informaes do goleiro
public
  X: integer;//Posio X do goleiro  (coluna)
  Y: integer;//Posicao Y do goleiro (linha)
  DX: integer;//Direao do movimento do goleiro
  DY: integer;
  constructor Create(AOwner: TComponent); override;
  destructor Destroy; override;
end;


TArquivoDados = class(TComponent)// Classe com as informaes do problema especfico.
public
  na: integer;//Numero de adversrios
  adversarios: TList;//Lista com os adversarios
  Objetivos: Tlist;
  EstadoInicial: TEstado;
  Objetivo: TEstado;
  bola: TBola;//Posio da Bola
  goleiro: TGoleiro; //Goleiro
  gol: TRect; //Posiao do Gol
  procedure AcharObjetivos;
  constructor Create(AOwner: TComponent); override;
  destructor Destroy; override;
end;

TNOH = class(TComponent)//Classe que representa um n do grafo de busca
public
  estado: TEstado;//Estado do n
  g: double;//Custo para chegar no n
  h: double;//Estimativa de custo para chegar ao objetivo
  f: double;//Estimativa de custo total para chegar ao objetivo
  melhorAntecessor: TNOH;//Ponteiro para o caminho usado para chegar ao n
  melhorSucessor:TNOH;
  sucessores: TList;//Lista dos sucessores do n
  procedure Atualizaf();//Atualiza o valor de f com base em g e h
  constructor Create(AOwner: TComponent); override;
  destructor Destroy; override;
end;

TBuscaA = class(TComponent)// Classe que representa o Algoritmo A*.
private
  procedure AtualizaG(NOH: TNOH);//Funo recursiva que atualiza o Valor de G dos sucessores.
  function EstaEm(Lista: TList; Estado: TEstado; g:double): integer;//Funo auxiliar que verifica
  //se um estado est em abertos ou fechados
  function EstadoMeta(estadoAtual: TEstado): boolean;//Verifica se um estado  o estado-meta
  procedure GerarSucessores(NohAtual: TNOH);//Gera os possveis sucessores do n
  function CalculaH(estadoAtual: TEstado; Objetivos: TList): double;//Clculo de H
  procedure PrepararListas(estadoInicial:TEstado);//Inicializa as listas abertos e fechados
public
  Abertos: TList;//Lista dos Ns Abertos
  Fechados: TList;//Lista dos Ns Fechados
  Problema: TArquivoDados;//Objeto com os dados do problema
  NohFinal: TNOH;
  function ExecutaBusca(Aproblema: TArquivoDados): integer;//Executa a busca A*
  constructor Create(AOwner: TComponent); override;
  destructor Destroy; override;
end;

  TForm1 = class(TForm)//Classe da Janela a ser usada
    OpenDialog1: TOpenDialog; BtnAbrir: TButton;
    Memo1: TMemo; Panel1: TPanel; Panel2: TPanel; Panel3: TPanel;
    Panel4: TPanel;
    Panel7: TPanel; Label2: TLabel; Label3: TLabel;
    Label4: TLabel; Label5: TLabel;
    GroupBox1: TGroupBox;
    GroupBox2: TGroupBox;
    CBRastroBola: TCheckBox;
    CBRastroAdversario: TCheckBox;
    CBRastroRobo: TCheckBox;
    GroupBox3: TGroupBox;
    BStart: TButton;
    ScrollBarVelocidade: TScrollBar;
    GBVelocidade: TGroupBox;
    CBPassoAPasso: TCheckBox;
    GBConfig: TGroupBox;
    LabelSintaxe: TLabel;
    Memo2: TMemo;
    BtnSalvar: TButton;
    Button1: TButton;
    SaveDialog1: TSaveDialog;
    GroupBox4: TGroupBox;
    CBAdversario: TCheckBox;
    GBPlan: TGroupBox;
    CBPlan: TCheckBox;
    BtnSobre: TButton;
    CBProtetor: TCheckBox;
    TimeOutEdit: TEdit;
    Label1: TLabel;
    CBAtaque: TCheckBox;
    GroupBox5: TGroupBox;
    ScrollBola: TScrollBar;
    Label6: TLabel;
    Label7: TLabel;
    Label8: TLabel;
    CBPoschute: TCheckBox;
    Label9: TLabel;
    Panel5: TPanel;
    GroupBox6: TGroupBox;
    CBGoleiro: TCheckBox;
    CBRastroGoleiro: TCheckBox;
    Label10: TLabel;
    Label11: TLabel;
    Label12: TLabel;
    SpinEdit1: TSpinEdit;
    BtnFim: TButton;
    procedure AtualizarCampo();
    procedure BtnAbrirClick(Sender: TObject);
    procedure FormCreate(Sender: TObject);
    procedure FormDestroy(Sender: TObject);
    procedure mostraResultado(Busca: TBuscaA; ArquivoDados: TArquivoDados) ;
    procedure BStartClick(Sender: TObject);
    procedure BtnEditarClick(Sender: TObject);
    procedure BtnSalvarClick(Sender: TObject);
    procedure Button1Click(Sender: TObject);
    procedure BtnSobreClick(Sender: TObject);
    procedure BtnFimClick(Sender: TObject);

  private
    { Private declarations }
  public
     Paineis: TPanels;
     nl: integer;
     nc: integer;
     TemGoleiro,FimPressed:boolean;
     procedure plot(X, Y:integer; cor:TColor);
    { Public declarations }
  end;
var
  Form1: TForm1;

implementation
{$R *.dfm}

function OrdenaAbertos(Item1, Item2: Pointer): Integer;
begin //Funo Auxiliar para ordenar a lista Abertos em funo de f
result:= round ((TNoh(Item1).f-TNoh(Item2).f)*1000);
end;

constructor TEstado.Create(AOwner: TComponent);
begin
  inherited Create(AOwner);
end;

destructor TEstado.Destroy;
begin
  inherited;
end;

constructor TBola.Create(AOwner: TComponent);
begin
  inherited Create(AOwner);
end;

destructor TBola.Destroy;
begin
  inherited;
end;

constructor TGoleiro.Create(AOwner: TComponent);
begin
  inherited Create(AOwner);
end;

destructor TGoleiro.Destroy;
begin
  inherited;
end;


procedure TNOH.Atualizaf;
begin
        f:=g+h;
end;

constructor TNOH.Create(AOwner: TComponent);
begin
        inherited Create(AOwner);
        estado:= nil;
        sucessores:= TList.Create;
        melhorAntecessor:= nil;
        g:=0;
        h:=0;
        f:=0;
end;

destructor TNOH.Destroy;
begin
 sucessores.Free;
  inherited;
end;

procedure TForm1.BtnAbrirClick(Sender: TObject);
begin
   if OpenDialog1.Execute then
   begin
     Memo1.Lines.Clear;//Le o Arquivo e monta a estrutura que vai ser usada no programa
     Memo1.Lines.LoadFromFile(OpenDialog1.FileName);
     AtualizarCampo();
   end;
end;

procedure TArquivoDados.AcharObjetivos;
var
        posicaoChute: TEstado;
        Dx:integer;
begin
//Verifica se chute reto faz gol
if (Self.bola.Y>= self.gol.Top) and (Self.bola.Y<=self.gol.Bottom) then
  begin
  posicaoChute:=TEstado.Create(self);
  posicaoChute.X:=self.bola.X-1;
  posicaoChute.Y:=self.bola.y;
  self.Objetivos.Add(posicaoChute);
  end;
Dx:=(self.gol.Left-self.bola.X) ;
//verifica se chute diagonal para baixo faz gol
if (Self.bola.Y+dx>= self.gol.Top) and (Self.bola.Y+dx<=self.gol.Bottom) then
  begin
  posicaoChute:=TEstado.Create(self);
  posicaoChute.X:=self.bola.X-1;
  posicaoChute.Y:=self.bola.y-1;
  self.Objetivos.Add(posicaoChute);
  end;
//verifica se chute diagonal para cima faz gol
if (Self.bola.Y-dx>= self.gol.Top) and (Self.bola.Y-dx<=self.gol.Bottom) then
  begin
  posicaoChute:=TEstado.Create(self);
  posicaoChute.X:=self.bola.X-1;
  posicaoChute.Y:=self.bola.y+1;
  self.Objetivos.Add(posicaoChute);
  end;
end;

constructor TArquivoDados.Create(AOwner: TComponent);
begin
  inherited Create(AOwner);
  estadoInicial:= TEstado.Create(AOwner);
  bola:=TBola.Create(AOwner);
  goleiro:=TGoleiro.Create(AOwner);
  adversarios:= TList.Create;//Lista com os adversarios
  Objetivos:= Tlist.Create;//Lista
  Objetivo:= TEstado.Create(AOwner);
end;

destructor TArquivoDados.Destroy;
var
        i:integer;
begin
  estadoInicial.Free;
  bola.Free;
  objetivo.Free;
  goleiro.Free;
  for  i:=0  to adversarios.Count-1  do
  begin
  TAdversario(Adversarios[i]).Free;
  end;
  adversarios.Free;
  for  i:=0  to objetivos.Count-1  do
  begin
  TEstado(objetivos[i]).Free;
  end;
  objetivos.Free;
  inherited;
end;


procedure TBuscaA.AtualizaG(NOH: TNOH);
var
        i:integer;
begin
        for  i:=0  to NOH.sucessores.Count-1 do
        begin
                if (Noh.g+1)< TNOH(Noh.sucessores.Items[i]).g then
                begin//Verifica se o g por esse caminho  melhor e atualiza e propaga a atualizacao
                     TNOH(Noh.sucessores.Items[i]).g:=Noh.g+1;
                     TNOH(Noh.sucessores.Items[i]).melhorAntecessor:=Noh;
                     TNOH(Noh.sucessores.Items[i]).Atualizaf;
                     AtualizaG(Noh.sucessores.Items[i]);
                end;
        end;
end;


function TBuscaA.CalculaH(estadoAtual: TEstado; Objetivos: TList): double;
var
        h:double;
        i:integer;
begin
        result:=10000;
        for  i:=0  to Objetivos.Count-1  do
        begin
                h:=abs(TEstado(Objetivos[i]).X-estadoAtual.X)+abs(TEstado(Objetivos[i]).Y-estadoAtual.Y);
                if h<result then result:=h;
        end;
end;


constructor TBuscaA.Create(AOwner: TComponent);
begin
  inherited Create(AOwner);
  Abertos:= nil;
  Fechados:= nil;
  Problema:=nil;
end;

destructor TBuscaA.Destroy;
var i:integer;
begin
Problema.Free;
if Abertos<>nil then
  begin
  for  i:=0  to Abertos.Count-1 do
       begin
       TNOH(Abertos.Items[i]).Free;
       end;
  Abertos.Free;
  Abertos:=nil;
  end;
if Fechados<>nil then
  begin
  for  i:=0  to Fechados.Count-1 do
       begin
       TNOH(Fechados.Items[i]).Free;
       end;
  Fechados.Free;
  Fechados:=nil;
  end;
  inherited;
end;

function TBuscaA.EstadoMeta(estadoAtual: TEstado): boolean;
begin//Se a posio do estado  igual  posio do objetivo retorna verdadeiro
        result:=false;
        if (Problema.Objetivo.X=estadoAtual.X) and (Problema.Objetivo.Y=estadoAtual.Y) then
                result:=true;
end;

function TBuscaA.EstaEm(Lista: TList; Estado: TEstado; g:double): integer;
var
        i:integer;
begin   //Se achar o estado na lista retorna a posio, seno retorna -1.
  result:=-1;
  for  i:=0  to Lista.Count-1  do
  begin
    if result=-1 then
    begin
      if ((TNoh(Lista.Items[i]).estado.X=Estado.X) and
      (TNoh(Lista.Items[i]).estado.Y=Estado.Y) and (TNoh(Lista.Items[i]).g=g))
      then
      begin
        result:=i;
      end;
    end;
  end;
end;

function TBuscaA.ExecutaBusca(Aproblema: TArquivoDados): integer;
var
        encontrou: boolean;
        NohExaminado: TNOH;
        i:integer;
        NohTemp: TNoh;
        indice: integer;
begin
  NohFinal:=nil;
  encontrou:=false;
  Problema:=AProblema;
  PrepararListas(Problema.estadoInicial);
  while (not(encontrou) and (abertos.Count<>0)) do
  begin//Repete at encontrar ou at no ter mais ns em abertos
  Abertos.Sort(@OrdenaAbertos);//Ordene Abertos
  NohExaminado:=abertos.First;
  Abertos.Delete(0);//Tira de abertos e coloca em fechados
  Fechados.Add(NohExaminado);
//   Form1.Paineis[NohExaminado.estado.X, NohExaminado.estado.Y].Color:=clPurple;//Tirar Depois
//   showmessage('continue');
  if EstadoMeta(NohExaminado.estado) then
  begin//Verifica se  estado meta
    encontrou:=true;
    NohFinal:=NohExaminado;
  end
  else
  begin//Gera os possveis sucessores
    GerarSucessores(NohExaminado);
    for  i:=0  to NohExaminado.sucessores.Count-1 do
    begin//Para cada sucessor verifica se est em abertos ou fechados
      if EstadoMeta(TNoh(NohExaminado.sucessores.Items[i]).estado) then
      begin//Verifica se  estado meta
        encontrou:=true;
        NohFinal:=NohExaminado.sucessores.Items[i];
        Abertos.Add(NohExaminado.sucessores.Items[i]);
//            Form1.Paineis[TNOH(NohExaminado.sucessores.Items[i]).estado.X, TNOH(NohExaminado.sucessores.Items[i]).estado.Y].Color:=clGray;
      end
      else
      begin
        indice := EstaEm(self.Abertos, TNoh(NohExaminado.sucessores.Items[i]).estado, TNoh(NohExaminado.sucessores.Items[i]).g);
        if indice <> -1 then
        begin//Se est em abertos verifica se o novo caminho  melhor e atualiza
          NohTemp:=NohExaminado.sucessores.Items[i];
          NohExaminado.sucessores.Items[i]:=Abertos.Items[indice];
          if NohTemp.g<TNoh(NohExaminado.sucessores.Items[i]).g then
          begin
            TNoh(NohExaminado.sucessores.Items[i]).g:=NohTemp.g;
            TNoh(NohExaminado.sucessores.Items[i]).melhorAntecessor:=NohExaminado;
            TNoh(NohExaminado.sucessores.Items[i]).Atualizaf;
          end;
          NohTemp.estado.Free;
          NohTemp.Free;
        end
        else
        begin //Se estiver em fechados verifica se o novo caminho  melhor e atualiza
          indice := EstaEm(Fechados, TNoh(NohExaminado.sucessores.Items[i]).estado, TNoh(NohExaminado.sucessores.Items[i]).g);
          if indice <> -1 then
          begin
            NohTemp:=NohExaminado.sucessores.Items[i];
            NohExaminado.sucessores.Items[i]:=Fechados.Items[indice];
            if NohTemp.g<TNoh(NohExaminado.sucessores.Items[i]).g then
            begin
              TNoh(NohExaminado.sucessores.Items[i]).g:=NohTemp.g;
              TNoh(NohExaminado.sucessores.Items[i]).melhorAntecessor:=NohExaminado;
              TNoh(NohExaminado.sucessores.Items[i]).Atualizaf;
              AtualizaG(NohExaminado.sucessores.Items[i]);
            end;
            NohTemp.estado.Free;
            NohTemp.Free;
          end
          else
          begin
            //Se no estiver nem em abertos nem em fechados coloca em abertos
            Abertos.Add(NohExaminado.sucessores.Items[i]);
//            Form1.Paineis[TNOH(NohExaminado.sucessores.Items[i]).estado.X, TNOH(NohExaminado.sucessores.Items[i]).estado.Y].Color:=clGray;
          end;
        end;
      end;
    end;
  end;
  end;
  if not(encontrou) then
    begin
      result:=0;
    end
    else
    begin
      //ShowMessage('SUCESSO!');
      result:=1;
    end;
  Problema:=nil;
end;

procedure TBuscaA.GerarSucessores(NohAtual: TNOH);
Var
  novoNoh: TNoh;
  novoEstado: TEstado;
  i,j,k: integer;
  valido: boolean;
begin//Tenta todos os possveis movimentos
  for  i:=-1  to 1 do
  begin
     for  j:=-1  to 1 do
     begin
       valido:=true;
       if   (NohAtual.estado.X+i=self.Problema.bola.X) and
            (NohAtual.estado.Y+j=self.Problema.bola.Y) and
            (NohAtual.h<>0) then valido:=false;
       //Verificar se tromba.
       for  k:=0  to self.Problema.adversarios.Count-1 do
       begin
         if (NohAtual.estado.X+i=TAdversario(Problema.adversarios[k]).X+(NohAtual.g+1)*TAdversario(Problema.adversarios[k]).DX) and
            (NohAtual.estado.Y+j=TAdversario(Problema.adversarios[k]).Y+(NohAtual.g+1)*TAdversario(Problema.adversarios[k]).DY)
            then valido:=false;
       //verifica Cruzamento.
         if (i*j=0) and
            (NohAtual.estado.X+i=TAdversario(Problema.adversarios[k]).X+(NohAtual.g)*TAdversario(Problema.adversarios[k]).DX) and
            (NohAtual.estado.X=TAdversario(Problema.adversarios[k]).X+(NohAtual.g+1)*TAdversario(Problema.adversarios[k]).DX) and
            (NohAtual.estado.Y+j=TAdversario(Problema.adversarios[k]).Y+(NohAtual.g)*TAdversario(Problema.adversarios[k]).DY) and
            (NohAtual.estado.Y=TAdversario(Problema.adversarios[k]).Y+(NohAtual.g+1)*TAdversario(Problema.adversarios[k]).DY)
            then valido:=false;
       end;
      //Verificar se o movimento  vlido.
       if (valido) then
          begin//Executa o movimento, criando um novo estado e um novo n
            novoNoh:=TNoh.Create(NohAtual.Owner);
            novoEstado:=TEstado.Create(NohAtual.Owner);
            novoEstado.X:=NohAtual.estado.X+i;
            novoEstado.Y:=NohAtual.estado.Y+j;
            novoNoh.estado:=novoEstado;
            novoNoh.g:=NohAtual.g + 1;
            novoNoh.h:=self.CalculaH(novoNoh.estado, problema.objetivos);
            novoNoh.atualizaf;
            novoNoh.melhorAntecessor:=NohAtual;
            NohAtual.sucessores.Add(novoNoh);
         end;
     end;
  end;
end;

procedure TBuscaA.PrepararListas(estadoInicial: TEstado);
Var
        Raiz: TNOH;
        i:integer;
begin
if Abertos<>nil then
  begin
  for  i:=0  to Abertos.Count-1 do
       begin
       TNOH(Abertos.Items[i]).Free;
       end;
  Abertos.Clear;
  end
  else Abertos:=TList.Create;
if Fechados<>nil then
  begin
  for  i:=0  to Fechados.Count-1 do
       begin
       TNOH(Fechados.Items[i]).Free;
       end;
  Fechados.Clear;
  end
  else Fechados:=TList.Create;
Raiz:=TNOH.Create(Owner);
Raiz.estado:=estadoInicial;
Raiz.h:=CalculaH(Raiz.estado, Problema.objetivos);
Raiz.g:=0;
Raiz.Atualizaf;
Abertos.Add(Raiz);
end;

procedure TForm1.FormCreate(Sender: TObject);
var
   i,j: integer;
begin
TimeOutEdit.Text:=inttostr(30);
SetLength(Paineis,COMPRIMENTO,LARGURA);
        for  i:=0  to COMPRIMENTO-1  do
        begin
          for  j:=0  to LARGURA-1  do
                begin
                Paineis[i, j]:=TPanel.Create(self);
                Paineis[i, j].Width:=15;
                Paineis[i, j].Height:=15;
                Paineis[i, j].Left:=15*i;
                Paineis[i, j].Top:=15*j;
                Paineis[i, j].Parent:=self.Panel1;
                Paineis[i, j].Color:=clGreen;
                end;
        end;
end;
{ TAdversario }

constructor TAdversario.Create(AOwner: TComponent);
begin
  inherited Create(AOwner);
end;

destructor TAdversario.Destroy;
begin
  inherited;
end;

procedure TForm1.FormDestroy(Sender: TObject);
var
   i,j: integer;
begin
        for  i:=0  to COMPRIMENTO-1  do
        begin
          begin
          for  j:=0  to LARGURA-1  do
                begin
                Paineis[i, j].Free;
                end;
          end;
        end;
end;

procedure TForm1.mostraResultado(Busca: TBuscaA; ArquivoDados: TArquivoDados);
var
  i,j, passos,aux,auxX,auxY,auxXA,auxXB,auxYA,auxYB:integer;
  Noh, Noh2:TNoh;
  Sucessor:TNoh;
//  dy:integer;
begin
  passos:=round(Busca.NohFinal.g);
  Sucessor:=Busca.NohFinal;
  repeat
    Noh:=Sucessor.melhorAntecessor;
    Noh.melhorSucessor:=Sucessor;
    Sucessor:=Noh;
  until Sucessor.melhorAntecessor=nil;

  //Mostrar Rota
  if CBPlan.Checked then
  begin
    Noh2:=Busca.NohFinal.melhorAntecessor;
    repeat
      Form1.Paineis[Noh2.estado.X, Noh2.estado.Y].Color:=clLime;
      Noh2:=Noh2.melhorAntecessor;
    until Noh2.melhorAntecessor=nil;
    showmessage('Rota Mostrada em Verde Claro');
    Noh2:=Busca.NohFinal.melhorAntecessor;
    repeat
      Form1.Paineis[Noh2.estado.X, Noh2.estado.Y].Color:=clGreen;
      Noh2:=Noh2.melhorAntecessor;
    until Noh2.melhorAntecessor=nil;
  end;

for  i:=0  to 1 do
begin

  //mover adversrios
  for  j:=0  to ArquivoDados.adversarios.Count-1 do
  begin

    if i>0 then
    if CBRastroAdversario.Checked then
      self.plot(TAdversario(ArquivoDados.adversarios[j]).X+ TAdversario(ArquivoDados.adversarios[j]).DX*(i-1),
      TAdversario(ArquivoDados.adversarios[j]).Y+ TAdversario(ArquivoDados.adversarios[j]).DY*(i-1), clPurple)
    else
      self.plot(TAdversario(ArquivoDados.adversarios[j]).X+ TAdversario(ArquivoDados.adversarios[j]).DX*(i-1),
      TAdversario(ArquivoDados.adversarios[j]).Y+ TAdversario(ArquivoDados.adversarios[j]).DY*(i-1), clGreen);

    self.plot(TAdversario(ArquivoDados.adversarios[j]).X+ TAdversario(ArquivoDados.adversarios[j]).DX*i,
    TAdversario(ArquivoDados.adversarios[j]).Y+ TAdversario(ArquivoDados.adversarios[j]).DY*i,
    clRed);
  end;

  //mover bola
    if i>0 then
    if CBRastroBola.Checked then
      self.plot(ArquivoDados.bola.X+ ArquivoDados.bola.DX*(i-1),
      ArquivoDados.bola.Y+ ArquivoDados.bola.DY*(i-1), clsilver)
    else
      self.plot(ArquivoDados.bola.X+ ArquivoDados.bola.DX*(i-1),
      ArquivoDados.bola.Y+ ArquivoDados.bola.DY*(i-1), clgreen);

    self.plot(ArquivoDados.bola.X+ArquivoDados.bola.DX*i,
    ArquivoDados.bola.Y+ArquivoDados.bola.DY*i,
    clwhite);
    if  (ArquivoDados.bola.X<=ArquivoDados.gol.Left) and
        (ArquivoDados.bola.X+ArquivoDados.bola.DX>=ArquivoDados.gol.Right)
        then
        begin
          auxXA:=ceil(ArquivoDados.bola.DX/2);
          auxXB:=floor(ArquivoDados.bola.DX/2);
          if ArquivoDados.bola.DY<0 then
          begin
            auxYA:=floor(ArquivoDados.bola.DY/2);
            auxYB:=ceil(ArquivoDados.bola.DY/2);
          end
          else
          begin
            auxYA:=ceil(ArquivoDados.bola.DY/2);
            auxYB:=floor(ArquivoDados.bola.DY/2);
          end;
          for aux:=0 to ArquivoDados.bola.DX do
          begin
             if ArquivoDados.bola.X+aux=ArquivoDados.gol.Left then auxX:=aux;
          end;
          if auxXA=auxX then auxY:=auxYA
          else auxY:=auxYB;
          self.plot(ArquivoDados.bola.X+auxX,ArquivoDados.bola.Y+auxY,clwhite);
        end;


//mover jogador

  if i>0 then
  if CBRastroRobo.Checked then
    self.plot(Noh.melhorAntecessor.estado.X, Noh.melhorAntecessor.estado.Y, clAqua)
  else
    self.plot(Noh.melhorAntecessor.estado.X, Noh.melhorAntecessor.estado.Y, clGreen);

  self.plot(Noh.estado.X, Noh.estado.Y, clBlue);


  Noh:=Noh.melhorSucessor;

  if CBPassoAPasso.Checked then showmessage('Prximo Passo!');
  application.ProcessMessages;
  sleep(5*ScrollBarVelocidade.Position);
end;
application.ProcessMessages;
end;

procedure TForm1.plot(X, Y: integer; cor: TColor);
begin
if (X>=0) and (Y>=0) and (X<COMPRIMENTO) and (Y<LARGURA) then
Paineis[X, Y].Color:=cor;
end;

procedure TForm1.BStartClick(Sender: TObject);
var
  novoAdversario: TAdversario;
  i,j,dy,ran,ran2,dir,cont,tempX,tempY,k,m,n,def,DelX,DelY,z,x,s:integer;
  valido,teste,defesa:boolean;
  final,final2:boolean;
  Dist,cons:double;
  Busca: TBuscaA;
  NextNoh: TNOH;
  ArquivoDados: TArquivoDados;
  Linha: TStringList;
begin
FimPressed:=False;
if Memo1.Lines.Count<>0 then
begin
cont:=0;
final:=false;
final2:=false;
defesa:=false;
Busca:= TBuscaA.Create(Owner);
Linha:= TStringList.Create;
ArquivoDados:= TArquivoDados.Create(self);
for  i:=0  to COMPRIMENTO-1  do
begin
  for  j:=0  to LARGURA-1  do
  begin
    Paineis[i, j].Color:=clGreen;
  end;
end;

if (ExtractStrings([' '], [], PChar(Memo1.Lines[0]), Linha) = 1) then
begin
  ArquivoDados.na:=strtoint(Linha[0]);
  Linha.Clear;
  if (ExtractStrings([' '], [], PChar(Memo1.Lines[1]), Linha) = 2) then //Robo
  begin
    ArquivoDados.EstadoInicial.X:=strtoint(Linha[0]);
    ArquivoDados.EstadoInicial.Y:=strtoint(Linha[1]);
    plot(ArquivoDados.EstadoInicial.X, ArquivoDados.EstadoInicial.Y, clblue);
  end;
  Linha.Clear;
  if (ExtractStrings([' '], [], PChar(Memo1.Lines[2]), Linha) = 2) then //Bola
  begin
    ArquivoDados.bola.X:=strtoint(Linha[0]);
    ArquivoDados.bola.Y:=strtoint(Linha[1]);
    ArquivoDados.bola.DX:=0;
    ArquivoDados.bola.DY:=0;
    ArquivoDados.Objetivo.X:=ArquivoDados.bola.X;
    ArquivoDados.Objetivo.Y:=ArquivoDados.bola.Y;
    plot(ArquivoDados.bola.X, ArquivoDados.bola.Y, clwhite);
  end;
  Linha.Clear;
  if (ExtractStrings([' '], [], PChar(Memo1.Lines[3]), Linha) = 2) then //Goleiro
  begin
    ArquivoDados.goleiro.X:=strtoint(Linha[0]);
    ArquivoDados.goleiro.Y:=strtoint(Linha[1]);
    TemGoleiro:=True;
    if ((ArquivoDados.goleiro.X=0) and (ArquivoDados.goleiro.Y=0)) or (CBGoleiro.Checked=false) then TemGoleiro:=False
    else begin
    ArquivoDados.goleiro.DX:=0;
    ArquivoDados.goleiro.DY:=0;
    plot(ArquivoDados.goleiro.X, ArquivoDados.goleiro.Y, clmaroon);
    end;
  end;
  Linha.Clear;
  if (ExtractStrings([' '], [], PChar(Memo1.Lines[4]), Linha) = 3) then //Gol
  begin
    ArquivoDados.gol.Left:=strtoint(Linha[0]);
    ArquivoDados.gol.Right:=ArquivoDados.gol.Left;
    ArquivoDados.gol.Top:=strtoint(Linha[1]);
    ArquivoDados.gol.Bottom:=ArquivoDados.gol.Top+strtoint(Linha[2])-1;
    for  j:=ArquivoDados.gol.Top  to ArquivoDados.gol.Bottom do
    begin
      plot(ArquivoDados.gol.Right, j, clBlack);
    end;
  end;
  for  i:=5 to ArquivoDados.na+4 do
  begin
    Linha.Clear;
    if (ExtractStrings([' '], [], PChar(Memo1.Lines[i]), Linha) = 4) then
    begin
      novoAdversario:=TAdversario.Create(self);
      novoAdversario.X:=strtoint(Linha[0]); //Posicao X
      novoAdversario.Y:=strtoint(Linha[1]); //Posicao Y
      novoAdversario.Dx:=strtoint(Linha[2]); //Orientacao
      novoAdversario.Dy:=strtoint(Linha[3]); //Orientacao
      ArquivoDados.adversarios.Add(novoAdversario);
      plot(novoAdversario.X, novoAdversario.Y,clred);
    end;
  end;
end;

ArquivoDados.AcharObjetivos;

if ArquivoDados.Objetivos.Count=0 then
begin
  ShowMessage('Configurao Invlida');
end
else
begin
  repeat
    if FimPressed then break;
    Busca.ExecutaBusca(ArquivoDados);
    mostraResultado(Busca, ArquivoDados);

    NextNoh:=Busca.NohFinal.melhorAntecessor;
    repeat
      if FimPressed then break;    
      NextNoh:=NextNoh.melhorAntecessor;
    until NextNoh.melhorAntecessor=nil;
    NextNoh:=NextNoh.melhorSucessor;

    ArquivoDados.EstadoInicial.X:=NextNoh.estado.X;
    ArquivoDados.EstadoInicial.Y:=NextNoh.estado.Y;

    Randomize;
    for  i:=0  to (ArquivoDados.na - 1)  do
    begin
      TAdversario(ArquivoDados.adversarios.Items[i]).X:= TAdversario(ArquivoDados.adversarios.Items[i]).X + TAdversario(ArquivoDados.adversarios.Items[i]).DX;
      TAdversario(ArquivoDados.adversarios.Items[i]).Y:= TAdversario(ArquivoDados.adversarios.Items[i]).Y + TAdversario(ArquivoDados.adversarios.Items[i]).DY;

      if CBAdversario.Checked then
      begin
        ran := floor(random(3))-1;
        sleep(20);
        TAdversario(ArquivoDados.adversarios.Items[i]).DX:= ran;
        randomize;
        ran := floor(random(3))-1;
        sleep(20);
        TAdversario(ArquivoDados.adversarios.Items[i]).DY:= ran
      end
      else if CBProtetor.Checked then
      begin
        tempX:=TAdversario(ArquivoDados.adversarios.Items[i]).DX;
        dir:=ArquivoDados.bola.x-(2*TAdversario(ArquivoDados.adversarios.Items[i]).DX+TAdversario(ArquivoDados.adversarios.Items[i]).X);
        if dir<0 then
             TAdversario(ArquivoDados.adversarios.Items[i]).DX:=-1
        else if dir>0 then
             TAdversario(ArquivoDados.adversarios.Items[i]).DX:=1
        else
           TAdversario(ArquivoDados.adversarios.Items[i]).DX:=0;
        tempY:=TAdversario(ArquivoDados.adversarios.Items[i]).DY;
        dir:=ArquivoDados.bola.y-(2*TAdversario(ArquivoDados.adversarios.Items[i]).DY+TAdversario(ArquivoDados.adversarios.Items[i]).Y);
        if dir<0 then
             TAdversario(ArquivoDados.adversarios.Items[i]).DY:=-1
        else if dir>0 then
             TAdversario(ArquivoDados.adversarios.Items[i]).DY:=1
        else
           TAdversario(ArquivoDados.adversarios.Items[i]).DY:=0;
        if i>=1 then
        begin
        teste:=false;
        repeat
          if FimPressed then break;
          if (((TAdversario(ArquivoDados.adversarios.Items[i-1]).Y+
          TAdversario(ArquivoDados.adversarios.Items[i-1]).DY)=
          (TAdversario(ArquivoDados.adversarios.Items[i]).Y+
          TAdversario(ArquivoDados.adversarios.Items[i]).DY)) and
          ((TAdversario(ArquivoDados.adversarios.Items[i-1]).X+
          TAdversario(ArquivoDados.adversarios.Items[i-1]).DX)=
          (TAdversario(ArquivoDados.adversarios.Items[i]).X+
          TAdversario(ArquivoDados.adversarios.Items[i]).DX)))
          then
          begin
            randomize;
            ran := floor(random(2))-1;
//            sleep(20);
//            TAdversario(ArquivoDados.adversarios.Items[i]).DY:= ran;
            TAdversario(ArquivoDados.adversarios.Items[i]).DY:= 0;
            randomize;
            ran := floor(random(2))-1;
//            sleep(20);
//            TAdversario(ArquivoDados.adversarios.Items[i]).DX:= ran;
            TAdversario(ArquivoDados.adversarios.Items[i]).DX:= 0;
          end
          else
            teste:=true;
        until teste;
        end;
      end
      else if CBAtaque.Checked then
      begin
        dir:=NextNoh.melhorSucessor.estado.X-(2*TAdversario(ArquivoDados.adversarios.Items[i]).DX+TAdversario(ArquivoDados.adversarios.Items[i]).X);
        if dir<0 then
           TAdversario(ArquivoDados.adversarios.Items[i]).DX:=-1
        else if dir>0 then
           TAdversario(ArquivoDados.adversarios.Items[i]).DX:=1
        else
           TAdversario(ArquivoDados.adversarios.Items[i]).DX:=0;
        dir:=NextNoh.melhorSucessor.estado.Y-(2*TAdversario(ArquivoDados.adversarios.Items[i]).DY+TAdversario(ArquivoDados.adversarios.Items[i]).Y);
        if dir<0 then
           TAdversario(ArquivoDados.adversarios.Items[i]).DY:=-1
        else if dir>0 then
           TAdversario(ArquivoDados.adversarios.Items[i]).DY:=1
        else
           TAdversario(ArquivoDados.adversarios.Items[i]).DY:=0;
        for z:=0 to (ArquivoDados.na-1) do
        begin
          for x:=0 to (ArquivoDados.na-1) do
          begin
            if x<>z then
            begin
            teste:=false;
            repeat
              if FimPressed then break;
              if (((TAdversario(ArquivoDados.adversarios.Items[x]).Y+
              TAdversario(ArquivoDados.adversarios.Items[x]).DY)=
              (TAdversario(ArquivoDados.adversarios.Items[z]).Y+
              TAdversario(ArquivoDados.adversarios.Items[z]).DY)) and
              ((TAdversario(ArquivoDados.adversarios.Items[z]).X+
              TAdversario(ArquivoDados.adversarios.Items[z]).DX)=
              (TAdversario(ArquivoDados.adversarios.Items[z]).X+
              TAdversario(ArquivoDados.adversarios.Items[z]).DX)))
              then
              begin
                randomize;
                ran := floor(random(2))-1;
//            sleep(20);
                TAdversario(ArquivoDados.adversarios.Items[z]).DY:= ran;
                randomize;
                ran := floor(random(2))-1;
//            sleep(20);
                TAdversario(ArquivoDados.adversarios.Items[z]).DX:= ran;
              end
              else
                teste:=true;
            until teste;
          end;
          end;
        end;
        end;

    end;
    if FimPressed then break;
    if (NextNoh.melhorSucessor.estado.X=ArquivoDados.Objetivo.X) and (NextNoh.melhorSucessor.estado.Y=ArquivoDados.Objetivo.Y) then
    begin
    //Mover a Bola
    self.plot(ArquivoDados.bola.X,ArquivoDados.bola.Y,clgreen);
    ArquivoDados.bola.DY:=ScrollBola.Position*(Busca.NohFinal.estado.Y-Busca.NohFinal.melhorAntecessor.estado.Y);
    ArquivoDados.bola.DX:=ScrollBola.Position*(Busca.NohFinal.estado.X-Busca.NohFinal.melhorAntecessor.estado.X);
    repeat
      if FimPressed then break;
      ArquivoDados.Objetivo.X:=ArquivoDados.bola.X+ArquivoDados.bola.DX;
      ArquivoDados.Objetivo.Y:=ArquivoDados.bola.Y+ArquivoDados.bola.DY;
      ArquivoDados.AcharObjetivos;
      if CBPoschute.Checked then Busca.ExecutaBusca(ArquivoDados);
      mostraResultado(Busca, ArquivoDados);
      ArquivoDados.bola.X:=ArquivoDados.bola.X+ArquivoDados.bola.DX;
      ArquivoDados.bola.Y:=ArquivoDados.bola.Y+ArquivoDados.bola.DY;

      if CBPoschute.Checked then
      begin
      NextNoh:=Busca.NohFinal.melhorAntecessor;
      repeat
        if FimPressed then break;
        NextNoh:=NextNoh.melhorAntecessor;
      until NextNoh.melhorAntecessor=nil;
      NextNoh:=NextNoh.melhorSucessor;

      ArquivoDados.EstadoInicial.X:=NextNoh.estado.X;
      ArquivoDados.EstadoInicial.Y:=NextNoh.estado.Y;
      end;

      if ((ArquivoDados.bola.X)>=ArquivoDados.gol.Left) then
      begin
       final:=true;
       final2:=true;
       application.ProcessMessages;
       Showmessage('GOOOOOOOOOOOLLLLLLLLLLL!');
       break;
      end;

      if TemGoleiro then
      begin

      if ((ArquivoDados.bola.X)>=ArquivoDados.gol.Left-SpinEdit1.Value) then
      begin
         def:=ArquivoDados.bola.X-ArquivoDados.goleiro.X;
         if def>0 then ArquivoDados.goleiro.DX:=1
         else if def<0 then ArquivoDados.goleiro.DX:=-1
         else ArquivoDados.goleiro.DX:=0;
         def:=ArquivoDados.bola.Y-ArquivoDados.goleiro.Y;
         if def>0 then ArquivoDados.goleiro.DY:=1
         else if def<0 then ArquivoDados.goleiro.DY:=-1
         else ArquivoDados.goleiro.DY:=0;
         if CBRastroGoleiro.Checked then                //Rastro Goleiro
           self.plot(ArquivoDados.goleiro.X,ArquivoDados.goleiro.Y,clolive)
         else
         self.plot(ArquivoDados.goleiro.X,ArquivoDados.goleiro.Y,clgreen);
         self.plot(ArquivoDados.goleiro.X+ArquivoDados.goleiro.DX,
                   ArquivoDados.goleiro.Y+ArquivoDados.goleiro.DY, clmaroon);
         ArquivoDados.goleiro.X:=ArquivoDados.goleiro.X+ArquivoDados.goleiro.DX;
         ArquivoDados.goleiro.Y:=ArquivoDados.goleiro.Y+ArquivoDados.goleiro.DY;


      //Detectar Defesa do Goleiro
       DelX:=ArquivoDados.bola.X+ArquivoDados.bola.DX-ArquivoDados.goleiro.X;
       DelY:=ArquivoDados.bola.Y+ArquivoDados.bola.DY-ArquivoDados.goleiro.Y;
       if ArquivoDados.bola.DX=1 then cons:=0.9
       else if ArquivoDados.bola.DX=3 then cons:=1.1
       else cons:=1;
       if ArquivoDados.bola.DY<>0 then
          Dist:=(Power((-DelX+(ArquivoDados.bola.DX)*(DelY)/ArquivoDados.bola.DY),2)/(power((ArquivoDados.bola.DX/ArquivoDados.bola.DY),2)+1))
       else
          Dist:=DelY*DelY;
       if (((DelX*DelX)+(DelY*DelY))<1) or ((Dist<1) and (abs(DelX)>cons)) then
       begin
         application.ProcessMessages;
         showmessage('Goleiro Defendeu');
         defesa:=true;
         break;
       end;

      end;
      end;
      Randomize;
      for  s:=0  to (ArquivoDados.na-1)  do
      begin
        TAdversario(ArquivoDados.adversarios.Items[s]).X:= TAdversario(ArquivoDados.adversarios.Items[s]).X + TAdversario(ArquivoDados.adversarios.Items[s]).DX;
        TAdversario(ArquivoDados.adversarios.Items[s]).Y:= TAdversario(ArquivoDados.adversarios.Items[s]).Y + TAdversario(ArquivoDados.adversarios.Items[s]).DY;
      end;

    until final;
    end;

    if defesa then break;

    cont:=cont+1;
    if cont>=strtoint(TimeOutEdit.Text) then
    begin
      showmessage('Fim de Jogo (Tempo Esgotado)');
      break;
    end;


    ArquivoDados.AcharObjetivos;
  until final2;

Busca.Free;
Linha.Free;
ArquivoDados.Free;
end;
end;
end;

procedure TForm1.BtnEditarClick(Sender: TObject);
begin
  Memo1.Enabled:= True;
end;

procedure TForm1.BtnSalvarClick(Sender: TObject);
begin
   if Memo1.Lines.Count<>0 then AtualizarCampo();
end;

procedure TForm1.AtualizarCampo();
var
novoAdversario: TAdversario;
i,j:integer;

Busca: TBuscaA;
ArquivoDados: TArquivoDados;
Linha: TStringList;

begin
Busca:= TBuscaA.Create(Owner);
Linha:= TStringList.Create;
ArquivoDados:= TArquivoDados.Create(self);
for  i:=0  to COMPRIMENTO-1  do
begin
  for  j:=0  to LARGURA-1  do
  begin
    Paineis[i, j].Color:=clGreen;
  end;
end;

if (ExtractStrings([' '], [], PChar(Memo1.Lines[0]), Linha) = 1) then
begin
  ArquivoDados.na:=strtoint(Linha[0]);
  Linha.Clear;
  if (ExtractStrings([' '], [], PChar(Memo1.Lines[1]), Linha) = 2) then //Robo
  begin
    ArquivoDados.EstadoInicial.X:=strtoint(Linha[0]);
    ArquivoDados.EstadoInicial.Y:=strtoint(Linha[1]);
    plot(ArquivoDados.EstadoInicial.X, ArquivoDados.EstadoInicial.Y, clblue);
  end;
  Linha.Clear;
  if (ExtractStrings([' '], [], PChar(Memo1.Lines[2]), Linha) = 2) then //Bola
  begin
    ArquivoDados.bola.X:=strtoint(Linha[0]);
    ArquivoDados.bola.Y:=strtoint(Linha[1]);
    ArquivoDados.Objetivo.X:=ArquivoDados.bola.X;
    ArquivoDados.Objetivo.Y:=ArquivoDados.bola.Y;
    plot(ArquivoDados.bola.X, ArquivoDados.bola.Y, clwhite);
  end;
  Linha.Clear;
  if (ExtractStrings([' '], [], PChar(Memo1.Lines[3]), Linha) = 2) then //Goleiro
  begin
    ArquivoDados.goleiro.X:=strtoint(Linha[0]);
    ArquivoDados.goleiro.Y:=strtoint(Linha[1]);
    TemGoleiro:=True;
    if ((ArquivoDados.goleiro.X=0) and (ArquivoDados.goleiro.Y=0)) or (CBGoleiro.Checked=false) then TemGoleiro:=False
    else begin
    ArquivoDados.goleiro.DX:=0;
    ArquivoDados.goleiro.DY:=0;
    plot(ArquivoDados.goleiro.X, ArquivoDados.goleiro.Y, clmaroon);
    end;
  end;
  Linha.Clear;
  if (ExtractStrings([' '], [], PChar(Memo1.Lines[4]), Linha) = 3) then //Gol
  begin
    ArquivoDados.gol.Left:=strtoint(Linha[0]);
    ArquivoDados.gol.Right:=ArquivoDados.gol.Left;
    ArquivoDados.gol.Top:=strtoint(Linha[1]);
    ArquivoDados.gol.Bottom:=ArquivoDados.gol.Top+strtoint(Linha[2])-1;
    for  j:=ArquivoDados.gol.Top  to ArquivoDados.gol.Bottom do
    begin
      plot(ArquivoDados.gol.Right, j, clBlack);
    end;
  end;
  for  i:=5  to ArquivoDados.na+4  do
  begin
    Linha.Clear;
    if (ExtractStrings([' '], [], PChar(Memo1.Lines[i]), Linha) = 4) then
    begin
      novoAdversario:=TAdversario.Create(self);
      novoAdversario.X:=strtoint(Linha[0]); //Posicao X
      novoAdversario.Y:=strtoint(Linha[1]); //Posicao Y
      novoAdversario.Dx:=strtoint(Linha[2]); //Orientacao
      novoAdversario.Dy:=strtoint(Linha[3]); //Orientacao
      ArquivoDados.adversarios.Add(novoAdversario);
      plot(novoAdversario.X, novoAdversario.Y,clred);
    end;
  end;
end;

ArquivoDados.AcharObjetivos;
{
for  i:=0  to  ArquivoDados.Objetivos.Count-1 do
begin
plot(TEstado(ArquivoDados.Objetivos[i]).X, TEstado(ArquivoDados.Objetivos[i]).Y, clLime);
end;
}

if ArquivoDados.Objetivos.Count=0 then
begin
  ShowMessage('Configurao Invlida.');
end
else
begin
  Busca.ExecutaBusca(ArquivoDados);
end;

Busca.Free;
Linha.Free;
ArquivoDados.Free;

end;


procedure TForm1.Button1Click(Sender: TObject);
begin
   if SaveDialog1.Execute then
   begin
     //Le o Arquivo e monta a estrutura que vai ser usada no programa
     Memo1.Lines.SaveToFile(SaveDialog1.FileName)
   end;
end;

procedure TForm1.BtnSobreClick(Sender: TObject);
begin
  Form2.ShowModal;
end;





procedure TForm1.BtnFimClick(Sender: TObject);
begin
        FimPressed:=True;
end;

end.
