% MATLAB script 5.x - file: LASER.M
% Metodo de treinamento: BP
% Programa valido apenas para 1 camada escondida
%   sem ligacoes diretas entre as camadas de entrada e de saida
% Assumindo que unidades escondidas e de saida usam funcao tanh(x)
%
% Rotinas auxiliares: bpc.m, calcer.m, nnout.m, ord.m
% Notacao: MATRIZES: todas as letras maiusculas
%          VetoreS: todos sao vetores coluna
%                   apenas letras inicial e/ou final sao maiusculas
%          elementos: todas as letras minusculas
%
% Autor: Cairo L. Nascimento Jr. - ITA - cairo@ita.br
% Data: 8/Nov/2002

% Preparacao para posicionamento das figuras
clear all; close all; Nf=1;
figure(Nf);   set(gcf,'Position',[  5  305   395   225]);
set(gcf,'DoubleBuffer','on'); % habilitar tracado mais rapido das linhas
                              % para mostrar animacoes em maquinas mais rapidas
figure(Nf+1); set(gcf,'Position',[405  305   395   225]);
set(gcf,'DoubleBuffer','on');
figure(Nf+2); set(gcf,'Position',[  5   35   395   195]);
set(gcf,'DoubleBuffer','on');


% Matriz INPD = entrada dos padroes a serem aprendidos
% Matriz OUTD = saida desejada para os padroes a serem aprendidos
% Cada padrao e' uma coluna
% numero de padroes = numero de colunas
% numero de unidades de entrada = numero de linhas

load A.dat; % vetor com 1000 linhas
Atrain=A;
LenData=length(A);
Njan=8; % tamanho da janela
for j=1:LenData-Njan;
    INPD(1:Njan,j)=A(j:j+Njan-1,1);
    OUTD(1,j)=A(j+Njan,1);
end;
maxdata=max(A);
mindata=min(A);
INPD=0.95*(2*INPD-maxdata-mindata)/(maxdata-mindata);
OUTD=0.95*(2*OUTD-maxdata-mindata)/(maxdata-mindata);

%INPD=[-1  1 -1  1
%      -1 -1  1  1];
%OUTD=[-1  1  1 -1];

% Descricao das variaveis globais
% Estas variaveis globais sao usadas nas rotinas auxliares
global iter npat nhid nout netah netao ninp netadec biasder nord;

nhid=5;       % numero de unidades escondidas
netah=0.08/4;  % learning rates para as 2 camadas de pesos
netao=0.04/4;
erromax=0.01;  % maximo erro rms permitido para parar o treinamento da rede neural
niter=60;    % numero maximo de iteracoes de treinamento
nord=1;       % nord = 0, > 0 : ordenacao sequencial ou aleatoria
netadec=0;    % decay rate
biasder=0.1;  % bias a ser adicionado na derivada da funcao de ativacao

% ninp = numero de unidades de entrada
% nout = numero de unidades de saida
% nhid = numero de unidades da camada escondida
% npat = numero de padroes a serem aprendidos pela rede
% nunits = numero total de unidades
[ninp,npat]=size(INPD);
[nout,npat]=size(OUTD);
nunits = ninp+nhid+nout;

% Matriz WHI = matriz de pesos entre as camadas de entrada e escondida
% Matriz WOH = matriz de pesos entre as camadas escondida  e de saida
% BiasH = vetores coluna com os bias da camada escondida
% BiasO = vetores coluna com os bias da camada de saida
% Notacao: W(i,j) = peso da unidade i vindo da unidade j
% Os pesos recebidos pela unidade i estao contidos na linha  i
% Os pesos enviados  pela unidade j estao contidos na coluna j
% Geracao da rede inicial = matrizes de pesos e vetores de bias
rand('state',sum(100*clock));
rand('state',2);
WHI=2*rand(nhid,ninp)-1;
WOH=2*rand(nout,nhid)-1;
BiasH=2*rand(nhid,1)-1;
BiasO=2*rand(nout,1)-1;

% Loop das iteracoes de treinamento
ndisp=2;   % periodo de atualizacao da saida grafica
iter=0;

while iter <= niter;

	  % Calculo do erro nesta iteracao
        [errop,OutpatE,OutpatH]=calcer(WHI,WOH,BiasH,BiasO,INPD,OUTD);      
        erro(iter+1,1)=errop;
        
      % Mostrar cabecalho adequado na janela de comandos se 1a iteracao
        if iter==0;
           OutpatE0=OutpatE;
        end;

      % Visualizacao grafica das saidas durante a aprendizagem
      % adaptar para o caso niter nao multiplo de ndisp
        if ndisp*fix(iter/ndisp)==iter | iter == 0 | errop < erromax;
           figure(1);
           plot([1:length(OUTD)],OUTD,'r',[1:length(OutpatE)],OutpatE,'k');
           xlabel('tempo'); ylabel('Saida');
           title(['Saida desejada e do modelo, iter = ' num2str(iter) '/' num2str(niter)]);
           drawnow;
           %
           figure(2); semilogy([0:length(erro)-1],erro,[0:length(erro)-1],erro,'*');
           xlabel('iteracao'); ylabel('Erro RMS');
           title(['Erro RMS durante treinamento = ' num2str(errop)]);
           YLim=get(gca,'YLim'); YLim=log10(YLim);
           YLim=[floor(YLim(1)) ceil(YLim(2))];
           YLim=10.^YLim;
           set(gca,'YLim',YLim);
           drawnow;
           ES=OUTD-OutpatE;
           fprintf('Iter = %g, Erro RMS, min, medio, max = %g, %g, %g, %g\n',iter, errop, min(ES), mean(ES), max(ES));
           %
           figure(3);
           plot(OUTD-OutpatE);
           set(gca,'YLim',[-2 2]);
           xlabel('tempo'); ylabel('Erro de aproximacao');
           title(['Erro medio = ' num2str(mean(OUTD-OutpatE))]); 
           drawnow;
           pause(1);
        end; % end "ndisp*fix(iter/ndisp)==iter | iter == 0;" 
        % Treinar a rede neural se necessario
        if errop > erromax;
     	   % Chamada da rotina Back Propagation
             [WHI,WOH,BiasH,BiasO]=bpc(WHI,WOH,BiasH,BiasO,INPD,OUTD);
        else;
            break;
        end;
         
        iter=iter+1;
         
end; % end "iter <= niter;"

% Mostre curva de aprendizado ao termino do aprendizado
  figure(2); semilogy([0:length(erro)-1],erro,[0:length(erro)-1],erro,'*');
  xlabel('iteracao'); ylabel('Erro RMS');
  title(['Erro RMS apos treinamento = ' num2str(errop)]);
  YLim=get(gca,'YLim'); YLim=log10(YLim);
  YLim=[floor(YLim(1)) ceil(YLim(2))];
  YLim=10.^YLim;
  set(gca,'YLim',YLim);
  drawnow;


% Teste de generalizacao  
figure(Nf+3); set(gcf,'Position',[ 405  35   395   195]);
set(gcf,'DoubleBuffer','on');
clear A;
load A.cont; % vetor com 9093 linhas
LenData=length(A);
LenData=200;
Njan=8; % tamanho da janela
for j=1:LenData-Njan;
    INPD2(1:Njan,j)=A(j:j+Njan-1,1);
    OUTD2(1,j)=A(j+Njan,1);
end;
%maxdata=max(A);
%mindata=min(A);
INPD2=0.95*(2*INPD2-maxdata-mindata)/(maxdata-mindata);
OUTD2=0.95*(2*OUTD2-maxdata-mindata)/(maxdata-mindata);
[ninp,npat]=size(INPD2);
[nout,npat]=size(OUTD2);
nunits = ninp+nhid+nout;

[errop,OutpatE,OutpatH]=calcer(WHI,WOH,BiasH,BiasO,INPD2,OUTD2);

figure(4); plot([1:length(OUTD2)],OUTD2,'r',[1:length(OutpatE)],OutpatE,'k'); 
xlabel('tempo'); ylabel('Saida');
title('Saida desejada e do modelo no conj. de teste');

ES=OUTD2-OutpatE;
fprintf('Erro no conj. de teste: RMS, min, medio, max = %g, %g, %g, %g\n',errop, min(ES), mean(ES), max(ES));

% EOF LASER.M
