;***********************************************************************
;*	Instituto Tecnologico de Aeronautica                           *
;*	Divisao de Engenharia Eletronica                               *
;*	Aluna:	Gabriela Werner Gabriel                                *
;*	Programa: MOVEA.ASM - Versao: 2.0TG                            *
;*	Data: 25/Outubro/2003                                          *
;***********************************************************************

;***********************************************************************
;*	DESCRICAO DO PROGRAMA: ESTE PROGRAMA RECEBE OS DADOS ENVIADOS, *
;*	GRAVA-OS NA POSICAO DE MEMORIA RESERVADA PARA TABELA DE MOVI-  *
;*	MENTOS, INICIADA EM 4500H, E REALIZA O MOVIMENTO DA PLATAFORMA *
;*	PROPRIAMENTE DITO.					       *
;*	NO DECORRER DO PROGRAMA SAO REALIZADOS VARIOS PUSH E POP PARA  *
;*	PILHA, DEVIDO AS ROTINAS DE BASIC-52 QUE ESTAO INSERIDAS NO    *
;*	PROGRAMA.						       *
;***********************************************************************

;***********************************************************************
;*	ALOCACAO DA MEMORIA:					       *
;*	4500H - 51FFH : MEMORIA PARA A TABELA DE MOVIMENTOS	       *
;*	5200H - 54FFH : MEMORIA DESTINADA PARA OS TEXTOS DO PROGRAMA   *
;*	5500H - 7FFFH : MEMORIA PARA O CODIGO			       *
;***********************************************************************

;***********************************************************************
;*	DEFINICAO DAS VARIAVEIS GERAIS DO PROGRAMA		       *
;***********************************************************************

		ACC	DATA	0E0H
		DPH	DATA	083H
		DPL	DATA	082H
		SP	DATA	081H

;***********************************************************************
;*	ALOCACAO DA PILHA					       *
;*	A PILHA TEM INICIO NA RAM PARA FINS GERAIS DO MICROCONTROLADOR *
;***********************************************************************

		MOV	SP,30H

;***********************************************************************
;*	DECLARACAO DAS VARIAVEIS UTILIZADAS NA RECEPCAO DE DADOS       *
;*	TH1 - BYTE ALTO, TIMER 1				       *
;*	TMOD - REGISTRADOR PARRA MODO DE OPERACAO DO TIMER	       *
;*	SBUF - BUFFER DA PORTA SERIAL				       *
;*	SCON - REGISTRADOR PARA CONTROLE DA PORTA SERIAL	       *
;*	IE - HABILITA INTERRUPCAO				       *
;*	PSW - PALAVRA QUE DEFINE O STATUS DE PROGRAMA		       *
;*	RCLK - SELECIONA O MODO DE RECEPCAO DE DADOS		       *
;*	TR1 - BIT ON/OFF DO TIMER 1				       *
;*	RI - BIT DE INTERRUPCAO PARA RECEPCAO PELA PORTA SERIAL        *
;*	IP - BIT QUE DEFINE A PRIORIDADE DAS INTERRUPCOES	       *
;***********************************************************************

		TH1     DATA    08DH
		TMOD    DATA    089H
		SBUF    DATA    099H
		SCON    DATA    098H
		IE      DATA    0A8H
		PSW     DATA    0D0H
		RCLK    BIT     0CDH
		TR1     BIT     08EH
		RI      BIT     098H
		IP	DATA	0B8H

;***********************************************************************
;*	DECLARACAO DAS VARIAVEIS PARA MOVIMENTO DA PLATAFORMA	       *
;*	MOTORL, MOTORR - BITS DE DIRECAO PARA O MOTOR ESQUERDO E DIREI-*
;*			 TO, RESPECTIVAMENTE			       *
;*	DRIVEL, DRIVER - BITS DE PULSO PARA OS MOTORES ESQUERDO E DI-  *
;*			 REITO, RESPECTIVAMENTE			       *
;*	NPULSE - NUMERO DE PULSOS RELATIVO AO MOVIMENTO A SER EXECUTADO*
;*	TIPO - TIPO DO MOVIMENTO				       *
;*	S_TIPO - SUB-TIPO DO MOVIMENTO				       *
;*	ROT_45 - GUARDA O NUMERO DE PULSOS A SER REALIZADO NUMA ROTACAO*
;*		 DE 45 GRAUS					       *
;*	ROT_90 - GUARDA O NUMERO DE PULSOS A SER REALIZADO NUMA ROTACAO*
;*		 DE 90 GRAUS					       *
;*	ROT_135 - GUARDA O NUMERO DE PULSOS A SER REALIZADO NUMA ROTA- *
;*		  CAO DE 135 GRAUS				       *
;*	TR_SIDE - GUARDA O NUMERO DE PULSOS A SER REALIZADO NUMA TRANS-*
;*		  LACAO NUM LADO DO LABIRINTO			       *
;*	TR_DIAG - GUARDA O NUMERO DE PULSOS A SER REALIZADO NUMA TRANS-*
;*		  LACAO NUMA DIAGONAL DO LABIRINTO                     *
;***********************************************************************

		MOTORL	BIT	094H
		DRIVEL	BIT	095H
		MOTORR	BIT	096H
		DRIVER	BIT	097H
		NPULSE	EQU	R7
		ROT_45	DATA	08H
		ROT_90	DATA	0AH
		ROT_135	DATA	0CH
		TR_SIDE	DATA	0EH
		TR_DIAG	DATA	10H
		TIPO	DATA	11H
		S_TIPO	DATA	12H

;***********************************************************************
;*	CONFIGURACAO DO ENDERECO DE ATENDIMENTO DA INTERRUPCAO DE DE-  *
;*	TECCAO DE COLISAO					       *
;***********************************************************************

		ORG	4013H		;INTERRUPCAO EXTERNA 1 INT1 DO
					;BASIC-52
		LJMP	COLISAO

;***********************************************************************
;*	CONFIGURACAO DO ENDERECO DA INTERRUPCAO DE RECEPCAO DE DADOS   *
;*	UTILIZANDO ROTINA BASIC-52 DISPONIVEL NO KIT AES-51	       *
;***********************************************************************

	        ORG     4023H    	;VETOR DE ATENDIMENTO DA INTER-
					;RUPCAO SERIAL DO BASIC-52
	        LJMP    INTR     	;JUMP PARA INTERRUPCAO   

;***********************************************************************
;*	DEFINICAO DOS TEXTOS APRESENTADOS NO DECORRER DO PROGRAMA      *
;*	TAIS TEXTOS DEVEM SER TERMINADOS PELO PONTO FINAL, INDICATIVO  *
;*	DO FIM DA STRING					       *
;***********************************************************************

		ORG	5200H
TEXTO1:		DB	'FIM TRANSMISSAO - ENTER P/ CONT.'
TEXTO2: 	DB	'ROTACAO ANTI-HORARIA.'
TEXTO3:		DB	'ROTACAO HORARIA.'
TEXTO4:		DB	'TRANSLACAO TRAS.'
TEXTO5:		DB	'TRANSLACAO FRENTE.'
TEXTO6:		DB	'ERRO: TIPO INVALIDO.'
TEXTO7:		DB	'ERRO: SUB-TIPO INVALIDO.'
TEXTO8:		DB	'FIM LABIRINTO - RESET P/ TERMIN.'
TEXTO9:		DB	'AGUARDANDO DADOS.'
TEXTO10:	DB	'PROGRAMA INTERROMPIDO - COLISAO.'

;***********************************************************************
;*	PARTE 1 - RECEPCAO DOS DADOS VIA PORTA SERIAL		       *
;***********************************************************************

		ORG     5500H
PARTE_1:	PUSH	DPL
		PUSH	DPH
		CALL	4100H
		MOV	DPTR, #TEXTO9
		CALL	LCD			;ESCREVE O TEXTO 9 NO LCD
		POP	DPH
		POP	DPL
		MOV     TMOD, #20H		;MODO AUTO-RELOAD
	        MOV     SCON, #50H		;PALAVRA 8-BIT BAUD RATE
		MOV     TH1, #0FDH		;BAUD RATE 9600
		CLR     RCLK			;SELECIONA TIMER 1
		SETB    TR1			;INICIA TIMER 1
		ORL     IE, #10010000B		;HABILITA INT SERIAL
		MOV	DPTR, #4500H		;CARREGA O INICIO DO ENDE-
						;RECO DA MEMORIA NO QUAL
						;DEVERA SER GRAVADA A TA-
						;BELA DE MOVIMENTOS
ESPERA:		JMP	ESPERA

;***********************************************************************
;*	PARTE 2 - TRATAMENTO DOS DADOS RECEBIDOS		       *
;*	ESTA PARTE DO PROGRAMA TRATA OS DADOS DE QUANTIDADE DE PULSOS  *
;*	RECEBIDOS. NO CASO, SAO RECEBIDOS BYTE POR BYTE, PORTANTO, PARA*
;*	COMPOR NUMEROS GRANDES E NECESSARIO COMPO-LOS NA RECEPCAO. EN- *
;*	TAO, PARA CADA VARIAVEL E RECEBIDO PRIMEIRO A CENTENA, DEPOIS A*
;*	DEZENA E POR UTIMO A UNIDADE				       *
;***********************************************************************

PARTE_2:	MOV	DPTR, #4500H		;ENDERECO INICIO TABELA
;DADO ROT_45, SEQUENCIA DE TRATAMENTO DE DADOS
		MOVX	A, @DPTR		;TOMA O 1o DADO DA TABELA
		CLR	C			;LIMPA CARRY
		SUBB	A,#30H			;TRNSFORMA ASCII EM ALG
		CALL	MUL100			;TRANSFORMA EM CENTENA
		MOV	ROT_45, A		;GUARDA NUMERO
		INC	DPTR
		MOVX	A, @DPTR		;TOMA O 2o DADO DA TABELA
		CLR	C			;LIMPA CARRY
		SUBB	A, #30H			;TRANSOMA ASCII EM ALG
		CALL	MUL10			;TRANSFOEMA EM DEZENA
		ADD	A, ROT_45		;SOMA NUMERO COM ANTERIOR
		MOV	ROT_45, A		;GUARDA NUMERO
		INC	DPTR
		MOVX	A, @DPTR		;TOMA O 3o DADO DA TABELA
		CLR	C			;LIMPA CARRY
		SUBB	A, #30H			;TRANSFORMA ASCII EM ALG
		ADD	A, ROT_45		;SOMA COM ANTERIORES
		MOV	ROT_45, A		;GUARDA RESULTADO
;DADO ROT_90, MESMA SEQUENCIA DE TRATAMENTO FEITA PARA ROT_45
		INC	DPTR
		MOVX	A, @DPTR
		CLR	C
		SUBB	A,#30H
		CALL	MUL100
		MOV	ROT_90, A
		INC	DPTR
		MOVX	A, @DPTR
		CLR	C
		SUBB	A, #30H
		CALL	MUL10
		ADD	A, ROT_90
		MOV	ROT_90, A
		INC	DPTR
		MOVX	A, @DPTR
		CLR	C
		SUBB	A, #30H
		ADD	A, ROT_90
		MOV	ROT_90, A
;DADO ROT_135, MESMA SEQUENCIA DE TRATAMENTO FEITA PARA ROT_45
		INC	DPTR
		MOVX	A, @DPTR
		CLR	C
		SUBB	A,#30H
		CALL	MUL100
		MOV	ROT_135, A
		INC	DPTR
		MOVX	A, @DPTR
		CLR	C
		SUBB	A, #30H
		CALL	MUL10
		ADD	A, ROT_135
		MOV	ROT_135, A
		INC	DPTR
		MOVX	A, @DPTR
		CLR	C
		SUBB	A, #30H
		ADD	A, ROT_135
		MOV	ROT_135, A
;DADO TR_SIDE, MESMA SEUENCIA DE TRATAMENTO EITA PARA ROT_45
		INC	DPTR
		MOVX	A, @DPTR
		SUBB	A,#30H
		CALL	MUL100
		MOV	TR_SIDE, A
		INC	DPTR
		MOVX	A, @DPTR
		CLR	C
		SUBB	A, #30H
		CALL	MUL10
		ADD	A, TR_SIDE
		MOV	TR_SIDE, A
		INC	DPTR
		MOVX	A, @DPTR
		CLR	C
		SUBB	A, #30H
		ADD	A, TR_SIDE
		MOV	TR_SIDE, A
;DADO TR_DIAG, MESMA SEUENCIA DE TRATAMENTO FEITA PARA ROT_45
		INC	DPTR
		MOVX	A, @DPTR
		CLR	C
		SUBB	A,#30H
		CALL	MUL100
		MOV	TR_DIAG, A
		INC	DPTR
		MOVX	A, @DPTR
		CLR	C
		SUBB	A, #30H
		CALL	MUL10
		ADD	A, TR_DIAG
		MOV	TR_DIAG, A
		INC	DPTR
		MOVX	A, @DPTR
		CLR	C
		SUBB	A, #30H
		ADD	A, TR_DIAG
		MOV	TR_DIAG, A

;ESPERA TECLA DO KIT PARA DAR INICIO AO MOVIMENTO DOS MOTORES
		CALL	4108H			;ROTINA BASIC-52

;***********************************************************************
;*	PARTE 3 - MOVIMENTO DOS MOTORES DE PASSO		       *
;***********************************************************************

PARTE_3:	ORL	IE, #10000100B		;HABILIDADE A INTERRUPCAO
						;INT1 EXTERNA
		ORL	IP, #00000100B		;DEFINE INT1 COMO DE MA-
						;XIMA PRIORIDADE. A INT1
						;E UTILIZADA PARA DETEC-
						;CAO DE COLISAO
		MOV	DPTR, #450FH		;POSICAO DE MEMORIA NO
						;QUAL ENCONTRA-SE O 1o
						;TIPO DA TABELA DE MOVIM
BEGIN:		MOVX	A, @DPTR		;TOMA O TIPO
		CLR	C			;LIMPA CARRY
		SUBB	A,#30H			;TRANSFORMA ASCII EM ALG
		MOV	TIPO, A			;CARREGA TIPO DO MOVIMENTO
		INC	DPTR
		MOVX	A, @DPTR		;TOMA O SUBTIPO DA TA-
						;BELA DE MOVIMENTO
		CLR	C			;LIMPA CARRY
		SUBB	A,#30H			;TRANSFORMA ASCII EM ALG
		MOV	S_TIPO, A		;CARREGA SUB-TIPO DO MOV
;TESTA SE O MOVIMENTO E O DE TRANSLACAO PARA FRENTE
FORW:		MOV	A, TIPO			;CARREGA O TIPO
		CJNE	A, #1, BACK		;TIPO NAO PARA FRENTE,
						;SALTA PARA O TESTE TRAS
		MOV	A, S_TIPO		;CARREGA O SUBTIPO
		CJNE	A, #1, HOR1		;TESTA SUB-TIPO DE TRANS-
						;LACAO NO LADO DO LABIRINT
		MOV	A, TR_SIDE		;SE SUBTIPO E 1 --> MO-
						;VIMENTO NUM LADO DO LABIR
		MOV	NPULSE, A		;CARREGA O NUMERO DE PULSOS
						;PARA UMA TRANSLACAO NUM
						;LADO
		JMP	FOR2			;IMPLEMENTA TRANSLACAO PARA
						;FRENTE
HOR1:		MOV	A, S_TIPO		;CARREGA O SUBTIPO
		CJNE	A, #2, ERRO1		;TESTA SUB-TIPO DE TRANS-
						;LACAO NA DIAGONAL DO
						;LABIRINTO
		MOV	A, TR_DIAG		;SE SUBTIPO E 2 --> MO-
						;VIMENTO NUMA DIAGONAL DO
						;LABIRINTO
		MOV	NPULSE, A		;CARREGA O NUMERO DE PULSOS
						;PARA UMA TRANSLACAO NUMA
						;DIAGONAL
		JMP	FOR2			;IMPLEMENTA TRANSLACAO PARA
						;FRENTE
ERRO1:		PUSH	DPL
		PUSH	DPH
		CALL	4100H			;LIMPA LCD - BASIC-52
		MOV	DPTR, #TEXTO6
		CALL	LCD			;ESCREVE TEXTO LCD
		CALL	4108H			;ESPERA QUE UMA TECLA DO KIT
						;SEJA PRESSIONADA
		POP	DPH
		POP	DPL
		RET
FOR2:		MOV	A, NPULSE
		PUSH	ACC
		PUSH	DPL
		PUSH	DPH
		CALL	4100H			;LIMPA LCD - BASIC-52
		MOV	DPTR, #TEXTO5
		CALL	LCD			;ESCREVE TEXTO LCD
		POP	DPH
		POP	DPL
		POP	ACC
		MOV	NPULSE, A
		CALL	FORWARD			;TRANSLACAO PARA FRENTE
		JMP	O_LINE			;LE PROXIMO TIPO
;TESTA SE O MOVIMENTO E O DE TRANSLACAO PARA TRAS. A SEQUENCIA DE CODIGO
;PARA OS DEMAIS MOVIMENTOS: TRAS, ROTACAO HORARIA E ROTACAO ANTI-HORARIA
;E SEMELHANTE A DE TRANSLACAO PARA FRENTE.
BACK:		MOV	A, TIPO
		CJNE	A, #2, RIGHT		;TIPO NAO PARA TRAS, SALTA
						;PARA ROTACAO SENT HORARIO
		MOV	A, S_TIPO
		CJNE	A, #1, HOR2		;TESTA SUB-TIPO DE TRANS-
						;LACAO NO LADO DO LABIRINT
		MOV	A, TR_SIDE
		MOV	NPULSE, A
		JMP	BAC2
HOR2:		MOV	A, S_TIPO
		CJNE	A, #2, ERRO2		;TESTA SUB-TIPO DE TRANS-
						;LACAO NA DIAGONAL NO LA-
						;BIRINTO
		MOV	A, TR_DIAG
		MOV	NPULSE, A
		JMP	BAC2
ERRO2:		PUSH	DPL
		PUSH	DPH
		CALL	4100H			;LIMPA LCD - BASIC-52
		MOV	DPTR, #TEXTO6
		CALL	LCD			;ESCREVE TEXTO LCD
		CALL	4108H
		POP	DPH
		POP	DPL
		RET
BAC2:		MOV	A, NPULSE
		PUSH	ACC
		PUSH	DPL
		PUSH	DPH
		CALL	4100H			;LIMPA LCD - BASIC-52
		MOV	DPTR, #TEXTO4
		CALL	LCD			;ESCREVE TEXTO LCD
		POP	DPH
		POP	DPL
		POP	ACC
		MOV	NPULSE, A
		CALL	BACKWARD		;TRANSLACAO PARA TRAS
		JMP	O_LINE			;LE PROXIMO TIPO
;TESTA SE O MOVIMENTO E O DE ROTACAO HORARIA
RIGHT:		MOV	A, TIPO
		CJNE	A, #3, LEFT		;TIPO NAO ROTACAO HORARIA,
						;SALTA PARA ROT ANTI-HORAR
		MOV	A, S_TIPO
		CJNE	A, #1, HOR3		;TESTA SUB-TIPO DE ROTACAO
						;DE 45 GRAUS EM TORNO DO
						;EIXO DA PLATAFORMA
		MOV	A, ROT_45
		MOV	NPULSE, A
		JMP	RIG2
HOR3:		MOV	A, S_TIPO
		CJNE	A, #2, HOR4		;TESTA SUB-TIPO DE ROTACAO
						;DE 90 GRAUS EM TORNO DO
						;EIXO DA PLATAFORMA
		MOV	A, ROT_90
		MOV	NPULSE, A
		JMP	RIG2
HOR4:		MOV	A, S_TIPO
		CJNE	A, #3, ERRO3		;TESTA SUB-TIPO DE ROTACAO
						;DE 135 GRAUS EM TORNO DO
						;EIXO DA PLATAFORMA
		MOV	A, ROT_135
		MOV	NPULSE, A
		JMP	RIG2
ERRO3:		PUSH	DPL
		PUSH	DPH
		CALL	4100H			;LIMPA LCD - BASIC-52
		MOV	DPTR, #TEXTO6
		CALL	LCD			;ESCREVE TEXTO LCD
		CALL	4108H
		POP	DPH
		POP	DPL
		RET
RIG2:		MOV	A, NPULSE
		PUSH	ACC
		PUSH	DPL
		PUSH	DPH
		CALL	4100H			;LIMPA LCD - BASIC-52
		MOV	DPTR, #TEXTO3
		CALL	LCD			;ESCREVE TEXTO LCD
		POP	DPH
		POP	DPL
		POP	ACC
		MOV	NPULSE, A
		CALL	CWISE			;ROTACAO HORARIA
		JMP	O_LINE			;LE PROXIMO TIPO
;TESTA SE O MOVIMENTO E O DE ROTACAO ANTI-HORARIA
LEFT:		MOV	A, TIPO
		CJNE	A, #4, ENDED		;TIPO NA ROT ANTI-HORARIA,
						;SALTA PARA FIM DO MOVIMEN
		MOV	A, S_TIPO
		CJNE	A, #1, HOR5		;TESTA SUB-TIPO DE ROTACAO
						;DE 45 GRAUS EM TORNO DO
						;EIXO DA PLATAFORMA
		MOV	A, ROT_45
		MOV	NPULSE, A
		JMP	LEF2
HOR5:		MOV	A, S_TIPO
		CJNE	A, #2, HOR6		;TESTA SUB-TIPO DE ROTACAO
						;DE 90 GRAUS EM TORNO DO	
						;EIXO DA PLATAFORMA
		MOV	A, ROT_90		
		MOV	NPULSE, A
		JMP	LEF2
HOR6:		MOV	A, S_TIPO
		CJNE	A, #3, ERRO4		;TESTA SUB-TIPO DE ROTACAO
						;DE 135 GRAUS EM TORNO DO
						;EIXO DA PLATAFORMA
		MOV	A, ROT_135
		MOV	NPULSE, A
		JMP	LEF2
ERRO4:		PUSH	DPL
		PUSH	DPH
		CALL	4100H			;LIMPA LCD - BASIC-52
		MOV	DPTR, #TEXTO7
		CALL	LCD			;ESCREVE TEXTO LCD
		CALL	4108H
		POP	DPH
		POP	DPL
		RET
LEF2:		MOV	A, NPULSE
		PUSH	ACC
		PUSH	DPL
		PUSH	DPH
		CALL	4100H			;LIMPA LCD - BASIC-52
		MOV	DPTR, #TEXTO2
		CALL	LCD			;ESCREVE TEXTO LCD
		POP	DPH
		POP	DPL
		POP	ACC
		MOV	NPULSE, A
		CALL	CCWISE			;ROTACAO ANTI-HORARIA
		JMP	O_LINE			;LE PROXIMO TIPO
;TESTA O FIM DA TABELA DE MOVIMENTOS
ENDED:		MOV	A, TIPO
		CJNE	A, #11H, ERRO5		;TESTA FIM DA TABELA DE
		LJMP	TERM			;MOVIMENTOS, QUE DEVE SER
						;TERMINADA POR 'A', 41H-30H
		RET
ERRO5:		PUSH	DPL
		PUSH	DPH
		CALL	4100H			;LIMPA LCD - BASIC-52
		MOV	DPTR, #TEXTO6
		CALL	LCD			;ESCREVE TEXTO LCD
		CALL	4108H
		POP	DPH
		POP	DPL
		RET

O_LINE:		INC	DPTR
		LJMP	BEGIN

;***********************************************************************
;*	ROTINA QUE IMPLEMENTA O MOVIMENTO PARA FRENTE, SEGUNDO A QUAN- *
;*	TIDADE DE PULSOS DEFINIDA POR NPULSE			       *
;***********************************************************************

FORWARD:	SETB	MOTORL			;ROTACAO DO MOTOR ES-
						;QUERDO PARA FRENTE
		SETB	MOTORR			;ROTACAO DO MOTOR DIREI-
						;TO PARA FRENTE
LOOPF:		CALL	PULSE			;ENVIA NPULSE PULSOS AOS
		DJNZ	NPULSE, LOOPF		;MOTOR
		SETB	DRIVEL
		SETB	DRIVER
		CALL	AMOVE			;ESPERA ENTRE MOVIMENTOS
		RET

;***********************************************************************
;*	ROTINA QUE IMPLEMENTA O MOVINETO PARA TRAS, SEGUNDO A QUANTI-  *
;*	DADE DE PULSOS DEFINIDA POR NPULSE			       *
;***********************************************************************

BACKWARD:	CLR	MOTORL			;ROTACAO DO MOTOR ES-
						;QUERDO PARA TRAS
		CLR	MOTORR			;ROTACAO DO MOTOR DIREI-
						;TO PARA TRAS
LOOPB:		CALL	PULSE			;ENVIA NPULSE PULSOS AOS
		DJNZ	NPULSE, LOOPB		;MOTOR
		SETB	DRIVER
		SETB	DRIVEL
		CALL	AMOVE			;ESPERA ENTRE MOVIMENTOS
		RET

;***********************************************************************
;*	ROTINA QUE IMPLEMENTA O MOVIMENTO DE ROTACAO HORARIA, SEGUNDO A*
;*	QUANTIDADE DE PULSOS DEFINIDA POR NPULSE		       *
;***********************************************************************

CWISE:		SETB	MOTORL			;ROTACAO DO MOTOR ES-
						;QUERDO PARA FRENTE
		CLR	MOTORR			;ROTACAO DO MOTOR DIREI-
						;TO PARA TRAS
LOOPCW:		CALL	PULSE			;ENVIA NPULSE PULSOS AOS
		DJNZ	NPULSE, LOOPCW		;MOTOR
		SETB	DRIVER
		SETB	DRIVEL
		CALL	AMOVE			;ESPERA ENTRE MOVIMENTOS
		RET

;***********************************************************************
;*	ROTINA QUE IMPLEMENTA O MOVIMENTO DE ROTACAO ANTI-HORARIA, SE- *
;*	GUNDO A QUANTIDADE DE PULSOS DEFINIDA POR NPULSE	       *
;***********************************************************************

CCWISE:		CLR	MOTORL			;ROTACAO DO MOTOR ES-
						;QUERDO PARA TRAS
		SETB	MOTORR			;ROTACAO DO MOTOR DIREI-
						;TO PARA FRENTE
LOOPCCW:	CALL	PULSE			;ENVIA NPULSE PULSOS AOS
		DJNZ	NPULSE, LOOPCCW		;MOTOR
		SETB	DRIVER
		SETB	DRIVEL
		CALL	AMOVE			;ESPERA ENTRE MOVIMENTOS
		RET

;***********************************************************************
;*	ROTINA QUE IMPLEMENTA O ENVIO DOS PULSOS AOS MOTORES DE PASSO. *
;*	ENTRE CADA PULSO E IMPLEMENTADA UM ESPERA DE FORMA QUE OS MO-  *
;*	TORES CONSIGAM RESPONDER SATISFATORIAMENTE A ESTES PULSOS. O   *
;*	FORMATO DA ONDA ENVIADA AOS MOTORES E ENTAO QUADRADA           *
;***********************************************************************

PULSE:		CLR	DRIVEL			;SETA BIT DE PULSO DO
						;MOTOR ESQUERDO
		CLR	DRIVER			;SETA BIT DE PULSO DO
						;MOTOR DIREITO
		CALL	WAIT			;TEMPO
		SETB	DRIVEL			;ZERA BIT DE PULSO DO
						;MOTOR ESQUERDO
		SETB	DRIVER			;ZERA BIT DE PULSO DO
						;MOTOR DIREITO
		CALL	WAIT			;TEMPO
		RET

;***********************************************************************
;*	ROTINA QUE IMPLEMENTA TEMPO DE APROXIMADAMENTE 1 SEGUNDO       *
;***********************************************************************

WAIT:		MOV	R2, #100
		MOV	R3, #255
		MOV	R4, #255
		DJNZ	R2, $
		DJNZ	R3, $-1
		DJNZ	R4, $-2
		RET

;***********************************************************************
;*	ROTINA QUE IMPLEMENTA ESPERA ENTRE MOVIMENTOS. ESTA ESPERA E   *
;*	QUE A IMPLEMENTADA POR WAIT, POIS AQUI PODE OCORRER INVERSAO   *
;*	NO SENTIDO DOS MOTORES					       *
;***********************************************************************

AMOVE:		MOV	R2, #255
		MOV	R3, #255
		MOV	R4, #255
		MOV	R1, #2
		DJNZ	R2, $
		DJNZ	R3, $-1
		DJNZ	R4, $-2
		DJNZ	R1, $-3
		RET

;***********************************************************************
;*	ROTINA QUE TRANSFORMA ALGARISMO EM DEZENA - LOOP DE MULTIPLICA-*
;*	CAO                                       		       *
;***********************************************************************

MUL10:		MOV	R3, #9
		MOV	R4, A
MULT1:		ADD	A, R4
		DJNZ	R3, MULT1
		RET

;***********************************************************************
;*	ROTINA QUE TRANSFORMA ALGARISMO EM CENTENA - LOOP DE MULTIPLI- *
;*	CACAO							       *
;***********************************************************************

MUL100:		MOV	R3, #99
		MOV	R4,A
MULT2:		ADD	A, R4
		DJNZ	R3, MULT2
		RET

;***********************************************************************
;*	ROTINA QUE ESCREVE STRING NO LCD DO KIT AES-51 - CORRESPONDE A *
;*	UM LOOP QUE ESCREVE BYTES QUE TERMINA QUANDO UM BYTE ESPECIFICO*
;*	E ENCONTRADO, NO CASO O PONTO FINAL			       *
;***********************************************************************

LCD:		MOVX	A, @DPTR		;CARREGA O BYTE DA SE-
						;QUENCIA APONTADO POR
						;DPTR
		MOV	21H, A
		INC	DPTR			;TOMA O PROXIMO BYTE
		CALL	4104H			;ROTINA QUE ENVIA DADO
						;PRESENTE NO ENDERECO 21H
						;PARA O LCD - BASIC-52
		CJNE	A, #2EH, LCD		;COMPARA SE O BYTE LIDO
						;CORRESPONDE AO PONTO
		RET

;***********************************************************************
;*	SUB-ROTINA DE INTERRUPCAO DE DETECCAO DE COLISAO - AQUI E UTI- *
;*	LIZADA A INTERRUPCAO INT1 PARA DETECTAR A COLISAO.	       *
;*	INICIALMENTE OS VALORES DE DPTR E DOS REGISTRADORES DE QUANTI- *
;*	DADE DE PULSOS SAO SALVOS NA PILHA, E AO FINAL DA INTERRUPCAO  *
;*	RESGATADOS						       *
;***********************************************************************

COLISAO:	MOV	A, NPULSE
		PUSH	ACC
		PUSH	DPH
		PUSH	DPL
		ORL	PSW, #18H		;UTILIZA O BANCO 3 DE
						;REGISTRADORES
		CALL	4100H			;LIMPA LCD - BASIC-52
		MOV	DPTR, #TEXTO10
		CALL	LCD			;IMPRIME TEXTO 10 NO LCD
		CALL	4108H			;ESPERA QUE UMA TECLA DO
						;KIT SEJA PRESSIONADA
;OS PROXIMOS CODIGOS RESTAURAM A TELA DO LCD, NA SAIDA DA SUBROTINA
		MOV	A, TIPO
		CJNE	A, #4, COL1
		MOV	DPTR, #TEXTO2
		JMP	ESC
COL1:		CJNE	A, #3, COL2
		MOV	DPTR, #TEXTO3
		JMP	ESC
COL2:		CJNE	A, #2, COL3
		MOV	DPTR, #TEXTO4
		JMP	ESC
COL3:		MOV	DPTR, #TEXTO5
ESC:		CALL	LCD			;IMPRIME O TEXTO DO MO-
						;VIMENTO QUE ESTAVA SEN-
						;DO REALIZADO ANTES DA
						;INTERRUPCAO NO LCD
		POP	DPL
		POP	DPH
		POP	ACC
		MOV	NPULSE, A
		CLR	RI			;RESTAURA RI PARA QUE
						;UMA NOVA INTERRUPCAO
						;POSSA SER CHAMADA
		POP	PSW			;RESTAURA O PSW, QUE E
						;SALVO NA PILHA AUTOMA-
						;TICAMENTE QUANDO UMA
						;INTERRUPCAO E SOLICI-
						;TADA
		RETI

;***********************************************************************
;*	SUB-ROTINA DE INTERRUPCAO DE RECEPCAO DOS DADOS - AQUI E UTILI-*
;*	ZADA A INTERRUPCAO SERIAL PARA IMPLEMANTAR A RECEPCAO DE DADOS.*
;*	A INTERRUPCAO SERIAL E DEFINIDA COMO PRIORIDADE DEFAULT	       *
;***********************************************************************

INTR:		ORL     PSW, #18H 		;HABILITA BANCO 3 DE RE-
						;GISTRADORES
		MOV     A, SBUF			;LEITURA DA PORTA SERIAL
		MOVX	@DPTR,A			;SALVA O DADO NO ENDERE-
						;CO DE MEMORIA APONTADO
						;POR DPTR
		INC	DPTR			;TOMA O PROXIMO ENDERECO
						;DE MEMORIA DA TABELA DE
						;MOVIMENTOS
		CJNE	A, #41H,PROX		;COMPARA O DADO RECEBIDO
						;COM O CARACTERE 'A' QUE
						;INDICA O TERMINO DA TA-
						;BELA DE MOVIMENTOS E 
						;TERMINA A RECEPCAO
		JMP	FIMR
PROX:		CLR     RI			;HABILITA PROX RECEPCAO
		POP     PSW     		;RESTAURA PSW GUARDADO
						;NA PILHA PELO BASIC-52
		RETI
;CASO TERMINADA A INTERRUPCAO O PROGRAMA AVISA NO LCD E SALTA PARA ROTI-
;NA DE EXECUCAO DO MOVIMENTO DA TRAJETORIA
FIMR:		CALL	4100H			;LIMPA LCD - BASIC-52
		MOV	DPTR, #TEXTO1
		CALL	LCD			;ESCREVE TEXTO 1 NO LCD
		LJMP	PARTE_2			
		RET

;***********************************************************************
;*	FINALIZACAO DO PROGRAMA E DO LABIRINTO			       *
;***********************************************************************

TERM:		PUSH	DPL
		PUSH	DPH
		CALL	4100H			;LIMPA LCD - BASIC-52
		MOV	DPTR, #TEXTO8
		CALL	LCD			;ESCREVE TEXTO 8 LCD
		CALL	4108H			;AGUARDA
		POP	DPH
		POP	DPL
		END				;DIRETIVA FIM DE PROGRAMA
