/*;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;Programa SEND.CPP - versao 2.00                                       ;
;                                                                      ;
;   Instituto Tecnologico de Aeronautica (ITA)                         ;
;   Divisao de Engenharia Eletronica                                   ;
;   Departamento de Sistemas e Controle                                ;
;                                                                      ;
;   Programa desenvolvido em projeto de bolsa de iniciacao cientifica  ;
;   fornecida pela FAPESP, e aprimorado como trabalho de graduacao.    ;
;                                                                      ;
;   Aluna: Gabriela Werner Gabriel (T. 03)                             ;
;                                                                      ;
;   Orientadores: Prof. CAIRO L. NASCIMENTO JR.                        ;
;		  Prof. EDUARDO HISASI YAGYU                           ;
;                                                                      ;
;   Descricao Sumaria: Programa responsavel por enviar dados para      ;
;		       a plataforma (SEND)                             ;
;   Versao: 2.00                                                       ;
;   Data: 25/Outubro/2003                                              ;
;                                                                      ;
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;*/


/*....................................................................*/
/*Definicoes que serao utilizadas ao longo do programa                */
/*						      	              */
/*Definicao da porta serial COM1 para saida de dados, aqui COM1BASE,  */
/* endereco 3F8h                                                      */
/*Definicao da linha de controle para 3FBh, COM1BASE+3 - endereco do  */
/* registrador controle	                                              */
/*Definicao da linha de controle para 3FDh, COM1BASE+5 - endereco do  */
/* registrador de status                                              */
/*....................................................................*/

#define	COM1BASE	0x3F8
#define	TXDATA		COM1BASE
#define	LCR		(COM1BASE+3)
#define	LSR		(COM1BASE+5)

/*....................................................................*/
/*Aqui sao definidas as bibliotecas necessarias para a compilacao do  */
/* programa 		                                              */
/*                                                                    */
/*Conio.h - Biblioteca necessaria para o uso do getch()               */
/*Stdio.h - Biblioteca necessaria para ouso do outportb()             */
/*....................................................................*/

#include	"conio.h"
#include	"dos.h"
#include	"stdio.h"

/*....................................................................*/
/*Definicoes das subrotinas utilizadas no decorrer do programa        */
/*                                                                    */
/*setup_serial - Subrotina que configura os parametros da porta serial*/
/* para transmissao de dados                                          */
/*send_number -  Subrotina que envia um dado para a porta de comunica-*/
/* cao serial                                                         */
/*le_arquivo - Subrotina que le dados do arquivo TABM_NEW.TXT gerado  */
/* pelo programa ROMEO e chama a subrotina send_number para enviar os */
/* dados lidos para o microcontrolador                                */
/*....................................................................*/

void	setup_serial(void);
void	send_number(int ch);
int	le_arquivo (void);

/*....................................................................*/
/*Aqui comeca a rotina principal                                      */
/*....................................................................*/

int	main(void)
{
	setup_serial();	//Configuracao da porta de saida de dados
	le_arquivo ();	//Subrotina que le o arquivo gerado por ROMEO
	return (0);
}

/*....................................................................*/
/*Rotina para configuracao da porta serial. A configuracao utilizada  */
/* e 8 bits de dados,1 bit de stop bit, 1 bit de paridade sendo a pa- */
/* ridade.                                                            */
/*....................................................................*/

void	setup_serial(void)
{
	outportb(LCR, 0x80);	//Seta todos os bits do endereco asso-
				//ciado ao LCR, no caso o 03FBh
	outportb(TXDATA, 0x0C);	//Envia o divisor de frequencia para o
				//endereco do LCR e seu subsequente,
				//ja que este e um dado de 16 bits.
				//Nao esquecer que o numero enviado e
				//em hexadecimal, mesmo nao estando
				//indicado
	outportb(TXDATA+1,0x00);//Este numero e enviado para a posicao
				//subsequente do LCR, para completar o
				//dado anterior que seria de 16 bits
	outportb(LCR, 0x03);    //Aqui sao enviadas as demais configu-
				//racoes para a transmissao, conforme
				//a descricao abaixo
}
/* O bit carregado no LCR corresponde a 00000011b, que corresponde a (do
bit menos significativo para o mais):
0 - acesso ao buffer TD/RD, 0 - sada normal
0 - sem stick bit, 0 - paridade par
0 - sem bit de paridade, 0 - 1 stop bit
11 - 8 bits de dados */

/*....................................................................*/
/*Rotina de leitura de arquivo. O arquivo lido e o TABM_NEW.TXT que e */
/*gerado pelo programa ROMEO.CPP, responsavel por calcular a melhor   */
/*trajetoria entre dois pontos dentro de um labirinto. Este programa  */
/*encontra-se em anexo.                                               */
/*A estrutura do arquivo TABM_NEW.TXT segue o seguinte padrao:        */
/*(Em que cada variavel e composta por tres bytes, em que um corres-  */
/*ponde a centena, outro a dezena e outro a unidade, nesta ordem)     */
/*1o, 2o e 3o bytes - ROT_45: numero de pulsos para ser realizada uma */
/* rotacao de 45 graus.                                               */
/*4o, 5o e 6o bytes - ROT_90: numero de pulsos para ser realizada uma */
/* rotacao de 90 graus.                                               */
/*7o, 8o e 9o bytes - ROT_135: numero de pulsos para ser realizada uma*/
/* rotacao de 135 graus.                                              */
/*10o, 11o e 12o bytes - TR_LADO: numero de pulsos para ser realizada */
/* uma translacao em um  lado referente as dimensoes do labirinto.    */
/*13o, 14o e 15o bytes - TR_DIAG: numero de pulsos para ser realizada */
/* uma translacao em uma diagonal referente as dimensoes do labirinto.*/
/*16o byte em diante: tipos e subtipos de movimento, na sequencia do  */
/* movimento sendo alternados os tipos e os subtipos, ate o final do  */
/* movimento desejado.                                                */
/*A tabela de tipos e subtipos corresponde a tabela apresentada a seguir:

*******************************************************************
* TIPO * MOVIMENTO            * SUBTIPO  * MOVIMENTO              *
*******************************************************************
*  1   * Frente               *    1     * Translacao-lado        *
*      *                      *    2     * Translacao-diagonal    *
*  2   * Tras                 *    1     * Translacao-lado        *
*      *                      *    2     * Translacao-diagonal    *
*  3   * Rotacao horaria      *    1     * 45 graus               *
*      *                      *    2     * 90 graus               *
*      *                      *    3     * 135 graus              *
*  4   * Rotacao anti-horaria *    1     * 45 graus               *
*      *                      *    2     * 90 graus               *
*      *                      *    3     * 135 graus              *
******************************************************************* */

int le_arquivo (void)
{

   /*Definicao das variaveis locais da sub_rotina le_arquivo*/
   FILE *pa;
	int ch, cont;
	float resto;

   /*Abertura do arquivo para leitura da tabela de movimentos*/
   cont=1;
	if((pa=fopen("TABM_NEW.txt","r"))==NULL)
	{
		printf("Nao foi possivel abrir o arquivo.");
		return 1 ;
	}
	ch=getc(pa);		//Le um caractere do arquivo

	/*Envia os dados imprimiveis para a porta serial*/

	while (ch!=EOF)
	{
		if (ch>=48 && ch<=57 || ch==65 || ch==97)
		{
			/*Laco que escreve na tela do microcomputador os
			dados enviados*/
			if (ch==97) ch=65;
			if (1<=cont<=15)
			{
				if (cont==1)
				{
					printf ("\nRotacao 45 graus = %c",ch);
				}
				if (cont==4)
				{
					printf ("\nRotacao 90 graus = %c",ch);
				}
				if (cont==7)
				{
					printf ("\nRotacao 135 graus = %c",ch);
				}
				if (cont==10)
				{
					printf ("\nTranslacao num lado = %c",ch);
				}
				if (cont==13)
				{
					printf ("\nTranslacao numa diagonal = %c",ch);
				}
				if (cont==2||cont==5||cont==8||cont==11||cont==14)
				{
					printf("%c",ch);
				}
				if (cont==3||cont==6||cont==9||cont==12||cont==15)
				{
					printf("%c pulsos",ch);
				}
			}
			if (cont==15) printf("\n\n Tabela de Movimentos:");
			if (cont>=16 && ch!=65)
			{
				resto=cont%2;
				if (resto==0.0) printf ("\n Tipo: %c ",ch);
				else printf("- Sub-tipo: %c",ch);
			}
	 send_number(ch);	//Envia o dado pela porta serial
         cont=cont+1;
		}
		ch=getc(pa);
	}
	printf("\n");
	fclose (pa);		//Fecha o arquivo TABM_NEW.TXT
   return (0);
}//Fim da subrotina le_arquivo

/*....................................................................*/
/*Esta subrotina e responsavel por enviar caracteres para a porta se- */
/*rial do microcomputador com as configuracoes pre-definidas em subro-*/
/*tina anterior                                                       */
/*....................................................................*/

void send_number(int ch)
{
	/*Definicao das variaveis locais pertencentes a esta subrotina*/
	char status;

	/*Laco que verifica se uma nova transmissao ja pode ser realizada,
	testando se o buffer da porta serial esta vazio ou nao. Caso nao
	esteja, a verificacao e repetida ate que a condicao desejada seja
	atingida*/
	do
	{
		status=inportb(LSR) & 0x40;
	}while (status!=0x40);

	/*Comando que envia o dado para porta serial*/
	outportb(TXDATA, (char) ch);
}//Chave de fim de subrotina

